<?php

namespace Drupal\commerce_mastercard\Plugin\Commerce\PaymentGateway;

use Drupal\commerce_order\Entity\OrderInterface;
use Drupal\commerce_order\Entity\OrderItem;
use Drupal\commerce_payment\Plugin\Commerce\PaymentGateway\OffsitePaymentGatewayBase;
use Drupal\commerce_product\Entity\ProductVariation;
use Drupal\Core\Form\FormStateInterface;
use Drupal\Core\Language\LanguageManagerInterface;
use Drupal\Core\Url;
use Symfony\Component\HttpFoundation\RedirectResponse;
use Symfony\Component\HttpFoundation\Request;

use Drupal\commerce_payment\PaymentMethodTypeManager;
use Drupal\commerce_payment\PaymentTypeManager;
use Drupal\commerce_price\MinorUnitsConverterInterface;
use Drupal\Component\Datetime\TimeInterface;
use GuzzleHttp\ClientInterface;
use Drupal\Core\Entity\EntityTypeManagerInterface;
use Symfony\Component\DependencyInjection\ContainerInterface;
use Symfony\Component\HttpFoundation\Response;
use Drupal\Core\Logger\LoggerChannelTrait;

/**
 * Provides the Mastercard offsite Checkout payment gateway.
 *
 * @CommercePaymentGateway(
 *   id = "mastercard_redirect_checkout",
 *   label = @Translation("Mastercard (Redirect to Mastercard website)"),
 *   display_label = @Translation("Mastercard"),
 *    forms = {
 *     "offsite-payment" = "Drupal\commerce_mastercard\PluginForm\McPaymentForm",
 *   },
 *   payment_method_types = {"credit_card"},
 *   credit_card_types = {
 *     "mastercard", "visa",
 *   },
 * )
 */
class RedirectCheckout extends OffsitePaymentGatewayBase {

  use LoggerChannelTrait;

  /**
   * Guzzle\Client instance.
   *
   * @var \GuzzleHttp\ClientInterface
   */
  protected $httpClient;

  /**
   * The language manager service.
   *
   * @var \Drupal\Core\Language\LanguageManagerInterface
   */
  protected $languageManager;

  /**
   * Constructs a new OffsiteRedirect object.
   *
   * @param array $configuration
   *   A configuration array containing information about the plugin instance.
   * @param string $plugin_id
   *   The plugin_id for the plugin instance.
   * @param mixed $plugin_definition
   *   The plugin implementation definition.
   * @param \Drupal\Core\Entity\EntityTypeManagerInterface $entity_type_manager
   *   The entity type manager.
   * @param \Drupal\commerce_payment\PaymentTypeManager $payment_type_manager
   *   The payment type manager.
   * @param \Drupal\commerce_payment\PaymentMethodTypeManager $payment_method_type_manager
   *   The payment method type manager.
   * @param \Drupal\Component\Datetime\TimeInterface $time
   *   The time.
   * @param \GuzzleHttp\ClientInterface $http_client
   *   The http client.
   * @param \Drupal\commerce_price\MinorUnitsConverterInterface $minor_units_converter
   *   The minor units converter.
   */
  public function __construct(array $configuration, $plugin_id, $plugin_definition, LanguageManagerInterface $language_manager, EntityTypeManagerInterface $entity_type_manager, PaymentTypeManager $payment_type_manager, PaymentMethodTypeManager $payment_method_type_manager, TimeInterface $time, ClientInterface $http_client, MinorUnitsConverterInterface $minor_units_converter = NULL) {
    parent::__construct($configuration, $plugin_id, $plugin_definition, $entity_type_manager, $payment_type_manager, $payment_method_type_manager, $time, $minor_units_converter);
    $this->httpClient = $http_client;
    $this->languageManager = $language_manager;
  }

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container, array $configuration, $plugin_id, $plugin_definition) {
    return new static(
      $configuration,
      $plugin_id,
      $plugin_definition,
      $container->get('language_manager'),
      $container->get('entity_type.manager'),
      $container->get('plugin.manager.commerce_payment_type'),
      $container->get('plugin.manager.commerce_payment_method_type'),
      $container->get('datetime.time'),
      $container->get('http_client'),
      $container->get('commerce_price.minor_units_converter'),
    );
  }

  /**
   * {@inheritdoc}
   */
  public function defaultConfiguration() {
    return [
      'merchant_link' => '',
      'merchant_id' => '',
      'password' => '',
      'interaction_operation' => '',
      'currency' => '',
    ] + parent::defaultConfiguration();
  }

  /**
   * {@inheritdoc}
   */
  public function buildConfigurationForm(array $form, FormStateInterface $form_state) {
    $form = parent::buildConfigurationForm($form, $form_state);

    $form['merchant_link'] = [
      '#type' => 'textfield',
      '#title' => t('Merchant link'),
      '#description' => t('The Merchant link of the account you want to use to receive real payments.'),
      '#default_value' => $this->configuration['merchant_link'],
      '#required' => TRUE,
    ];

    $form['merchant_id'] = [
      '#type' => 'textfield',
      '#title' => t('Merchant ID'),
      '#description' => t('The Merchant ID of the account you want to use to receive real payments.'),
      '#default_value' => $this->configuration['merchant_id'],
      '#required' => TRUE,
    ];

    $form['password'] = [
      '#type' => 'textfield',
      '#title' => $this->t('API Password'),
      '#description' => $this->t('The API password for the account you want to use to receive real payments.'),
      '#default_value' => $this->configuration['password'],
      '#required' => TRUE,
    ];

    $form['interaction_operation'] = [
      '#type' => 'select',
      '#title' => $this->t('Interaction operation'),
      '#description' => $this->t('Indicates the operation that you wish to perform during the Hosted Checkout interaction.'),
      '#default_value' => $this->configuration['interaction_operation'],
      '#required' => TRUE,
      '#options' => [
        'AUTHORIZE' => t('AUTHORIZE'),
        'PURCHASE' => t('PURCHASE'),
      ],
    ];

    $form['currency'] = [
      '#type' => 'textfield',
      '#title' => $this->t('Order currency'),
      '#description' => $this->t('The Order currency for the account you want to use to receive real payments.'),
      '#default_value' => $this->configuration['currency'],
      '#required' => TRUE,
    ];

    return $form;
  }

  /**
   * {@inheritdoc}
   */
  public function submitConfigurationForm(array &$form, FormStateInterface $form_state) {
    parent::submitConfigurationForm($form, $form_state);
    $values = $form_state->getValue($form['#parents']);
    $this->configuration['merchant_link'] = $values['merchant_link'];
    $this->configuration['merchant_id'] = $values['merchant_id'];
    $this->configuration['password'] = $values['password'];
    $this->configuration['interaction_operation'] = strtoupper($values['interaction_operation']);
    $this->configuration['currency'] = strtoupper($values['currency']);
  }

  /**
   * {@inheritdoc}
   */
  public function onCancel(OrderInterface $order, Request $request) {
    $order->getState()->applyTransitionById('place');
    $order->save();
    $order->getState()->applyTransitionById('cancel');
    $order->save();

    $order_item = OrderItem::load($order->get('order_items')->getString());
    $url = \Drupal::request()->getSchemeAndHttpHost();
    $variation_id = $order_item->purchased_entity->target_id;
    $variation = ProductVariation::load($variation_id);
    // Get id of product.
    $product_id = $variation->product_id->getString();

    if (isset($product_id)) {
      // Retrieve node related with product.
      $campaign_node = \Drupal::entityTypeManager()->getStorage('node')->loadByProperties(['field_product' => $product_id]);
      if (isset($campaign_node)) {
        $campaign_node = reset($campaign_node);
        $node_id = $campaign_node->get('nid')->getString();

        // Get current language.
        $languages = $this->languageManager->getCurrentLanguage();
        // Get the current URL based on language.
        $url = Url::fromRoute('entity.node.canonical', ['node' => $node_id], ['absolute' => TRUE])
          ->setOption('language', $languages)->toString();
      }
    }
    $this->messenger()->addError($this->t('The order is canceled.'));
    $response = new RedirectResponse($url);
    $response->send();
  }

  /**
   * {@inheritdoc}
   */
  public function onReturn(OrderInterface $order, Request $request) {
    $tempstore = \Drupal::service('tempstore.private')->get('commerce_mastercard');
    $successIndicator = $tempstore->get('successIndicator');
    $resultIndicator = $request->query->get('resultIndicator');
    if (!empty($resultIndicator)) {
      if ($resultIndicator == $successIndicator) {
        $current_state = $order->getState()->getId();
        $payment = $this->entityTypeManager->getStorage('commerce_payment')->loadMultipleByOrder($order);
        $payment = reset($payment);
        $payment->set('state', 'complete');
        $payment->save();
        if ($current_state == 'draft') {
          // Set the order state to completed by applying the
          // 'validate' transition.
          $order->getState()->applyTransitionById('place');
          $order->save();
          $order->getState()->applyTransitionById('validate');
          $order->save();
          return new Response("Completed", 200);
        }
        else {
          $this->messenger()->addError($this->t('The order is not validated.'));
          return new Response('The order is already validated.', 200);
        }
      }
      else {
        // Set the order state to canceled by applying the
        // 'cancel' transition.
        $order->getState()->applyTransitionById('cancel');
        $order->save();
        $this->messenger()->addError($this->t('The order is failed.'));
        return new Response('The order is failed.', 200);
      }
    }
  }

}
