<?php

namespace Drupal\commerce_mastercard\PluginForm;

use Drupal\commerce_payment\PluginForm\PaymentOffsiteForm as BasePaymentOffsiteForm;
use Drupal\Core\Config\ConfigFactoryInterface;
use Drupal\Core\DependencyInjection\ContainerInjectionInterface;
use Drupal\Core\Entity\EntityTypeManager;
use Drupal\Core\Form\FormStateInterface;
use Drupal\commerce_product\Entity\ProductVariation;
use Drupal\Core\Language\LanguageManagerInterface;
use Symfony\Component\DependencyInjection\ContainerInterface;

/**
 * {@inheritdoc}
 */
class McPaymentForm extends BasePaymentOffsiteForm implements ContainerInjectionInterface {

  /**
   * The payment gateway live environment URL.
   *
   * @var string
   */
  const PAYMENT_LIVE_URL = 'https://ap-gateway.mastercard.com/api/nvp/version/61';

  /**
   * The payment gateway test environment URL.
   *
   * @var string
   */
  const PAYMENT_TEST_URL = 'https://test-gateway.mastercard.com/api/nvp/version/61';

  /**
   * Contains the configuration object factory.
   *
   * @var \Drupal\Core\Config\ConfigFactoryInterface
   */
  protected $configFactory;

  /**
   * The language manager service.
   *
   * @var \Drupal\Core\Language\LanguageManagerInterface
   */
  protected $languageManager;

  /**
   * The entity type manager.
   *
   * @var \Drupal\Core\Entity\EntityTypeManager
   */
  protected $entityTypeManager;

  /**
   * {@inheritdoc}
   */
  public function __construct(ConfigFactoryInterface $config_factory, LanguageManagerInterface $language_manager, EntityTypeManager $entity_type_manager) {
    $this->configFactory = $config_factory;
    $this->languageManager = $language_manager;
    $this->entityTypeManager = $entity_type_manager;
  }

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container) {
    return new static(
      $container->get('config.factory'),
      $container->get('language_manager'),
      $container->get('entity_type.manager')
    );
  }

  /**
   * {@inheritdoc}
   */
  public function buildConfigurationForm(array $form, FormStateInterface $form_state) {
    $form = parent::buildConfigurationForm($form, $form_state);
    // Get the language of page.
    $language = $this->languageManager->getCurrentLanguage()->getId();
    /** @var \Drupal\commerce_payment\Entity\PaymentInterface $payment */
    $payment = $this->entity;
    // Save the payment entity so that we can get its ID and use it for
    // Then, when user returns from the off-site redirect, we will update
    // the same payment.
    $payment->save();

    // Order data.
    /** @var \Drupal\commerce_order\Entity\OrderInterface $order */
    $order = $payment->getOrder();
    $order_id = $order->get('order_id')->getString();
    $order_items = $order->getItems();
    $product_variation_id = $order_items[0]->get('purchased_entity')->getString();
    $product_variation = ProductVariation::load($product_variation_id);
    $product = $product_variation->getProduct();
    $product_id = $product_variation->product_id->getString();
    $price = $product_variation->getPrice()->getNumber();
    if (isset($product_id)) {
      // Retrieve node related with product.
      $campaign_node = $this->entityTypeManager->getStorage('node')->loadByProperties(['field_product' => $product_id]);
      if (isset($campaign_node)) {
        $campaign_node = reset($campaign_node);
        $campaign_name = $campaign_node->get('title')->getString();
        if ($language == 'ar') {
          $campaign_name = $campaign_node->getTranslation('ar')->getTitle();
        }
        $data['campaign_name'] = $campaign_name;
      }
    }

    // Payment data.
    /** @var \Drupal\commerce_payment\Plugin\Commerce\PaymentGateway\OffsitePaymentGatewayInterface $payment_gateway_plugin */
    $payment_gateway_plugin = $payment->getPaymentGateway()->getPlugin();
    $configuration = $payment_gateway_plugin->getConfiguration();
    $merchant_id = $configuration['merchant_id'];
    $data['merchant_id'] = $merchant_id;
    $password = $configuration['password'];
    // Add the payment id to the transaction id to help in loading.
    $transaction_id = rand(1000, 9999) . $payment->id();
    $payment->set('remote_id', $transaction_id);
    $payment->save();

    // Set the order state to validation by applying the
    // 'place' transition.
    $order->set('state', 'draft');
    $order->set('field_checkout_language', $language);
    $order->save();

    // Prepare Mastercard payment gateway parameters.
    $payment_redirect_url = $configuration['merchant_link'];
    $data['mode'] = $configuration['mode'];
    $data['apiOperation'] = 'CREATE_CHECKOUT_SESSION';
    $data['apiUsername'] = "merchant.$merchant_id";
    $data['merchant'] = $merchant_id;
    $data['interaction_operation'] = $configuration['interaction_operation'];
    $data['order_id'] = $transaction_id . $order_id;
    $data['order_currency'] = strtoupper($configuration['currency']);
    $data['order_amount'] = number_format($price, 2);
    $data['interaction_returnUrl'] = $form['#return_url'];
    $data['interaction_cancelUrl'] = $form['#cancel_url'];

    // Load the site name out of configuration.
    $config = $this->configFactory->get('system.site');
    $data['site_name'] = $config->get('name');

    $ch = curl_init($payment_redirect_url);
    curl_setopt($ch, CURLOPT_POST, TRUE);
    curl_setopt($ch, CURLOPT_RETURNTRANSFER, TRUE);
    curl_setopt($ch, CURLOPT_POSTFIELDS, "apiOperation=CREATE_CHECKOUT_SESSION&apiPassword=$password&apiUsername=$data[apiUsername]&merchant=$data[merchant]&order.id=$data[order_id]&interaction.operation=$data[interaction_operation]&interaction.returnUrl=$data[interaction_returnUrl]&interaction.cancelUrl=$data[interaction_cancelUrl]&order.amount=$data[order_amount]&order.currency=$data[order_currency]");

    $headers = [];
    $headers[] = 'Content-Type: application/x-www-form-urlencoded';
    curl_setopt($ch, CURLOPT_HTTPHEADER, $headers);

    $result = curl_exec($ch);
    if (curl_errno($ch)) {
      echo 'ERROR:' . curl_error($ch);
    }
    $status = explode("=", explode("&", $result)[1])[1];
    curl_close($ch);
    if ($status == 'SUCCESS') {
      $successIndicator = explode("=", explode("&", $result)[5])[1];
      $data['successIndicator'] = $successIndicator;

      $tempstore = \Drupal::service('tempstore.private')->get('commerce_mastercard');
      $tempstore->set('successIndicator', $successIndicator);

      $sessionid = explode("=", explode("&", $result)[2])[1];
      $data['sessionid'] = $sessionid;

      // From the contact details.
      $data['Language'] = strtoupper(\Drupal::languageManager()->getCurrentLanguage()->getId());
    }

    return $this->buildRedirectForm(
      $form,
      $form_state,
      $payment_redirect_url,
      $data,
      self::REDIRECT_GET
    );
  }

  /**
   * Builds the redirect form.
   *
   * @param array $form
   *   The plugin form.
   * @param \Drupal\Core\Form\FormStateInterface $form_state
   *   The current state of the form.
   * @param string $redirect_url
   *   The redirect url.
   * @param array $data
   *   Data that should be sent along.
   * @param string $redirect_method
   *   The redirect method (REDIRECT_GET or REDIRECT_POST constant).
   *
   * @throws \Drupal\commerce\Response\NeedsRedirectException
   *   The redirect exception, if $redirect_method is REDIRECT_GET.
   */
  protected function buildRedirectForm(array $form, FormStateInterface $form_state, $redirect_url, array $data, $redirect_method = self::REDIRECT_GET) {
    if ($redirect_method == self::REDIRECT_GET) {
      $form['#attached']['library'][] = $data['mode'] == 'test' ? 'commerce_mastercard/checkout-test' : 'commerce_mastercard/checkout-live';
      $form['#attached']['drupalSettings']['commerce_mastercard'] = $data;
      return $form;
    }
  }

}
